from pandas.io.parsers.readers import TextFileReader as TextFileReader
from pandas.io.parsers.readers import TextParser as TextParser
from pandas.io.parsers.readers import read_csv as read_csv
from pandas.io.parsers.readers import read_fwf as read_fwf
from pandas.io.parsers.readers import read_table as read_table